<?php
App::uses('AppHelper', 'View/Helper');

// 文字列を取り扱うヘルパー

class StringHelper extends AppHelper
{
	// マルチバイト文字列を、指定されたエンコーディングに基づき「1文字」ずつに分解し配列にして返す。
	// 与えられた空文字列の場合は NULL を返す。
	public function mb_str_to_array($str, $encoding=NULL)
	{
		if ( is_null($encoding) )
		{
			$encoding = mb_internal_encoding();
		}
		$limit = mb_strlen($str, $encoding);
		$ret = NULL;
		for ( $i=0; $i<$limit; $i++ )
		{
			$ret[] = mb_substr($str, $i, 1, $encoding);
		}
		return $ret;
	}


	private function __is_ascii($char)
	{
		return ord($char) <= 127;
	}

	private function __is_control($char)
	{
		return ord($char) < 0x20;
	}
	
	private function __is_space($char)
	{
		return ord($char) == 0x20;
	}

	private function __get_token(&$char_array)
	{
		if ( ($tmp = current($char_array)) === FALSE )
		{
			// 空配列または配列末尾に達していた場合は FALSE を返す
			return FALSE;
		}
		if ( !$this->__is_ascii($tmp) )
		{
			// マルチバイト文字 (日本語前提)
			$next = next($char_array);
			if ( in_array($next, array('。', '、')) )
			{
				next($char_array);
				return $tmp . $next;
			}
			return $tmp;
		}
		else if ( $this->__is_space($tmp) )
		{
			// 空白文字は1つにまとめる
			for ( $next = next($char_array); $this->__is_space($next); $next = next($char_array) )
			{
				;	// 空白は読み飛ばす
			}
			return $tmp;
		}
		else
		{
			$ret = array($tmp);	// 最初の文字をプッシュ
			while ( ($tmp=next($char_array)) !== FALSE )
			{
//var_dump($tmp);
				if ( $this->__is_ascii($tmp) )
				{
					$ret[] = $tmp;	// ASCII 文字である間はプッシュ
				}
				else
				{
					break;	// ASCII 文字でなくなったらループから離脱
				}
			}
			return implode('', $ret);
		}
	}

	// シンプル版
	private function __mb_folding_sub($line, $width, $encoding)
	{
		$array = $this->mb_str_to_array($line, $encoding);
//debug(compact('line', 'width', 'array'));
		reset($array);
		$ret = array();
		$current_len = $tk_len = 0;
		$tmp = array();
		for ( $current_len = 0; current($array) !== FALSE; $current_len += $tk_len )
		{
			$tk = $this->__get_token($array);
			$tk_len = mb_strwidth($tk);
			if ( ($current_len + $tk_len) <= $width )
			{
				$tmp[] = $tk;
			}
			else
			{
				// 既定の長さに達したのでこれまでの文字を行として出力し、次の行の準備をする
				$ret[] = implode('', $tmp);
				if ( $this->__is_space($tk) )
				{
					// 行頭が空白文字の場合はスキップ
					$tmp = array();
					$tk_len = 0;
				}
				else
				{
					$tmp = array($tk);
				}
				$current_len = 0;
//debug(array('status' => '次の行', 'tmp' => $tmp, 'current_len' => $current_len));
			}
		}
		if ( count($tmp) > 0 )
		{
			$ret[] = implode('', $tmp);
		}
		return $ret;
	}

	public function mb_folding($str, $width, $encoding=NULL)
	{
		if ( is_null($encoding) )
		{
			// $encoding が指定されていない時の既定値
			$encoding = mb_internal_encoding();
		}
		mb_regex_encoding($encoding);
		$ret = array();	// 整形した行を保持する配列
		foreach ( mb_split('\r\n|\n|\r', $str) as $line )
		{
//debug(array('len' => mb_strwidth($line, $encoding), 'width' => $width));
			// 1行ごとに処理
			if ( mb_strwidth($line, $encoding) < $width )
			{
				// 出力幅が規定値以下の行はそのままプッシュ
				$ret[] = $line;
			}
			else
			{
				// 規定値より大きい行は分割してプッシュ
				foreach ( $this->__mb_folding_sub($line, $width, $encoding) as $l )
				{
					$ret[] = $l;
				}
			}
		}
//var_dump($ret);
		return $ret;
	}
}
?>